/*

Copyright (C) 2003 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef __libnd_tcpstate_h
#define __libnd_tcpstate_h

#include <libnd.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
  LND_TCP_ERROR            =  0,
  LND_TCP_LISTEN           =  1,
  LND_TCP_CLOSED_NORMAL    =  2,
  LND_TCP_RST_WAIT         =  3,
  LND_TCP_CLOSED_RST       =  4,
  LND_TCP_SYN_SENT         =  5,
  LND_TCP_SYN_ACK_SENT     =  6,
  LND_TCP_ESTABLISHED      =  7,
  LND_TCP_SHUTDOWN         =  8,
  LND_TCP_TIME_WAIT        =  9,
  LND_TCP_CLOSED_TIMEOUT   = 10,
} LND_TCPState;

/* Opaque structure for TCP connection state tracking */
typedef struct lnd_tcp_conn LND_TCPConn;

typedef void (*LND_TCPConnFunc)(LND_TCPConn *conn, void *user_data);


LND_TCPConn    *libnd_tcpconn_new(const LND_Packet *packet);
void            libnd_tcpconn_free(LND_TCPConn *tcpc);

void            libnd_tcpconn_state_update(LND_TCPConn *tcpc,
					   const LND_Packet *packet);

LND_TCPState    libnd_tcpconn_state_get(const LND_TCPConn *tcpc);
const char     *libnd_tcpconn_state_get_string(LND_TCPState state);

gboolean        libnd_tcpconn_handshake_seen(LND_TCPConn *tcpc);
gboolean        libnd_tcpconn_teardown_seen(LND_TCPConn *tcpc);
gboolean        libnd_tcpconn_is_dead(const LND_TCPConn *tcpc);

void            libnd_tcpconn_get_src(const LND_TCPConn *tcpc,
				      struct in_addr *ip_src,
				      guint16 *th_sport);
void            libnd_tcpconn_get_dst(const LND_TCPConn *tcpc,
				      struct in_addr *ip_dst,
				      guint16 *th_dport);
void            libnd_tcpconn_to_string(const LND_TCPConn *tcpc,
					char *str, int strlen);
void            libnd_tcpconn_data_set(LND_TCPConn *tcpc,
				       const char *key,
				       void *data);
void           *libnd_tcpconn_data_get(const LND_TCPConn *tcpc,
				       const char *key);
void           *libnd_tcpconn_data_remove(LND_TCPConn *tcpc,
					  const char *key);

GHashTable     *libnd_tcpconn_table_new(void);
void            libnd_tcpconn_table_free(GHashTable *ht);

void            libnd_tcpconn_table_add(GHashTable *ht, LND_TCPConn *tcpc);

LND_TCPConn    *libnd_tcpconn_table_lookup(GHashTable *ht,
					   const LND_Packet *packet);

void            libnd_tcpconn_table_remove(GHashTable *ht, LND_TCPConn *tcpc);

void            libnd_tcpconn_table_foreach(GHashTable *ht,
					    LND_TCPConnFunc func,
					    void *user_data);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
